/*
Arquivo:	fft.c
Descricao: 	Funcao para computar a transformada direta de Fourier (FFT)
		a partir dos dados no dominio do tempo.
Autor:		Joao Dovicchi
Data:		05/05
*/


#include "dft.h"

extern int isign;
extern long nn;
extern double *data;

void four1()
{
	unsigned long n, nmax, m, j, step, i;
	double wtemp, wr, wpr, wpi, wi, theta;
	float tempr, tempi;

		/*************************************/
		/* Efetua o algoritmo de bit-reverso */
		/*************************************/
	n=nn << 1;
	j=1;
	for( i=1; i<n; i+=2 )
	{
		if( j > i )
		{
			SWAP(data[j],data[i]);
			SWAP(data[j+1],data[i+1]);
		}
		m=n >> 1;
		while( m >= 2 && j > m)
		{
			j -= m;
			m >>= 1;
		}
		j += m;
	}

		/***************************************/
		/* Algoritmo de Danielson-Lanczos ver: */
		/* Numerical Recipes pag. 504          */
		/***************************************/
	nmax = 2;
	while( n > nmax )
	{
		step = nmax << 1;
		theta = isign*2*pi/nmax;
		wtemp = sin( 0.5*theta );
		wpr = -2.0*wtemp*wtemp;
		wpi=sin( theta );
		wr = 1.0;
		wi = 0.0;
		for( m=1; m<nmax; m+=2 )
		{
			for( i=m; i<=n; i+=step )
			{
				j = i + nmax;
				tempr = wr * data[j] - wi * data[j+1];
				tempi = wr * data[j+1] + wi * data[j];
				data[j] = data[i] - tempr;
				data[j+1] = data[i+1] - tempi;
				data[i] += tempr;
				data[i+1] += tempi;
			}
			wr = (wtemp=wr) * wpr - wi * wpi + wr;
			wi = wi * wpr + wtemp * wpi + wi;
		}
		nmax = step;
	}
}
